%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% fminit			                  	fminit	           					fminit
% =========================================================================  
% Genetic Algorithms Tools for MathTools 
% =========================================================================  
%                             Copyright (c) 1999, 2000
%                       Flexible Intelligence Group (FIG)       	             
%                   P. O. Box 861477, Tuscaloosa, AL 35486, USA.            
% =========================================================================  
%  PURPOSE   
%         Initialize GA variables
% .........................................................................   
%  GIVEN
% gap(1)	= 	Type of GA
%				1= Regular GA
%				2= Micro-GA
%				3=Steady State GA	 
% gap(2)	=	# of generations for evolution
% gap(3)	=	population size
% gap(4)	=	No of peaks;     
% gap(5)	=	Type of selection desired
%				1= Roulette Wheel Selection
%				2= Tournament Selection
%				3= Ranking Selection
% gap(6)	=	Tournament size (> 1 if Tournament selection is chosen).
% gap(7)	=	Steady State population size (>1 if Steady State GA is chosen)
% gap(8)	=	Probability of Crossover (0 - 1)   
% gap(9)	= 	Number of crossover points ( > 1)
% gap(10)	=	Probability of Mutation (0 - 1) 
% gap(11)	= 	Probability of Gaussian Mutation (0 - 1)	[OPTIONAL]
% gap(12)	=	Micro-GA inner loop # of Generations   	[OPTIONAL]	
% gap(13)	=	# of asexual children for stochastic coding	[OPTIONAL]
% ...........................................................................   
%  RETURNS
% gap		= 	GA Parameters
% fpd		=	function parameter details (fpd)
% fud		=	Function Details (fud)
% mean_table	=	table of mean values
% sigma_table	=	table of s.d.
% pot_table	=	power of two table
%.............................................................................
%   USES  
% =========================================================================   
%  SYNTAX  /  USAGE   
%  [gap,fpd,fud,mean_table,sigma_table,pot_table]=fminit(P_min,P_max,P_res,P_type,gap)
%  ========================================================================   
%  CODE    

function [gap,fpd,fud,mean_table,sigma_table,pot_table]=fminit(P_min,P_max,P_res,P_type,gap)

%  SHARED DATA 
%Make sure that size(P_min)=size(P_max)=size(P_res)=size(P_type);

tempin(1)	= gap(1);
tempin(2)	= gap(2);
tempin(3)	= gap(3);
tempin(4)	= gap(4);
tempin(5)	= gap(5);
tempin(6)	= gap(6);
tempin(7)	= gap(7);
tempin(8)	= gap(8);
tempin(9)	= gap(9);
tempin(10)	= gap(10);
pssparam(1)=2;
[pssparam(2), pssparam(3)]=size(tempin);
[temp]=ftgalib(pssparam, tempin);
Nga_choice	= temp(1);
Ngen		= temp(2);
Npop		= temp(3);
Noofpeaks	= temp(4);
Nselect		= temp(5);
Tsize		= temp(6);
Nsspop		= temp(7);
pcross		= temp(8);
Ncross		= temp(9);
pmutate		= temp(10);


if max(size(gap)) < 11
   gap(11) = 0.0;		% Gaussian Mutation probability;  
   gap(12)=1;		% Micro-ga inner-loop
   if Nga_choice==2, gap(12)=5; end  
   gap(13)=3;		% #of asexual children for stochastic coding 
end

if max(size(gap)) < 12
   gap(12)=1;		% Micro-ga inner-loop
   if Nga_choice==2, gap(12)=5; end  
   gap(13)=3;		% #of asexual children for stochastic coding
end

if max(size(gap)) < 13
   gap(13)=3;		% #of asexual children for stochastic coding
end

if (size(P_type,1)>1)
  P_min=P_min';
  P_max=P_max';
  P_type=P_type';
  P_res=P_res';
end

Noofobj=1;	% Number of objectives (Could be changed later on to include  more than 1 objective)
mm=1;		% mm=1 for minimization; mm=-1 for maximization (size=Noofobj)

max_Nbits=20;	%maximum # of bits per parameter

pot_table=2.^(0:1:(max_Nbits-1));

Noofp=max(size(P_type));
max_rows=512;

%------------------------------------------------------------  
% pc,pslope,pres,Nbits,mean_table,sigma_table,etc.   
%------------------------------------------------------------
for j=1:Noofp
  if P_type(j) == 1			%Integer Type
      Nbits(j)=ceil(log2(P_max(j)-P_min(j)+1));
  elseif P_type(j) == 2		%Real Type with discrete values
     Nbits(j)=ceil(log2((P_max(j)-P_min(j))/P_res(j)));
  else 							%Real Type with continuous values
      Nbits(j)=P_type(j);     
  end
  max_rows=2.^max(Nbits);
  mean_table=zeros(max_rows,Noofp);
  sigma_table=zeros(max_rows,Noofp);
end

for j=1:Noofp
  if P_type(j) == 1	%Integer Type
      pc(j)=P_min(j);
      pslope(j)=(P_max(j)-P_min(j))/(2^Nbits(j)-1);
  end

  if P_type(j) == 2		%Real Type with discrete values
     pc(j)=P_min(j);
     pslope(j)=P_res(j);
     pslope(j)= (P_max(j)-P_min(j))/(2^Nbits(j)-1);
  end

  if P_type(j) > 2		%Real Type with continuous values
    for i=1:2^(P_type(j))
      pc(j)=0;
      pslope(j)=(P_max(j)-P_min(j))/(2^P_type(j)-1);
      mean_table(i,j)=P_min(j)+(i-1)*pslope(j);
      sigma_table(i,j)=pslope(j)/4;
    end
  end
end
Lchrom=sum(Nbits);

%------------------------------------------------------------  
%Function Parameters Details (fpd)
%------------------------------------------------------------
fpd(1,:)=P_type;
fpd(2,:)=P_min;
fpd(3,:)=P_max;
fpd(4,:)=P_res;
fpd(5,:)=Nbits;
fpd(6,:)=pc;
fpd(7,:)=pslope;

%-------------------------------------------------------------------------  
%Function Details (fud). fud (1-96) is kept empty for later use
%-------------------------------------------------------------------------
fud(1:96)=zeros(1,96);
fud(97)=Lchrom;
fud(98)=Noofp;
fud(99)=Noofobj;
fud(100)=mm;
%=========================================================================   
% (c) Flexible Intelligence Group (FIG)   
% END fminit.m   
% fminit			                  	fminit	           					fminit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

